# Jack and Jill in Python
# Purpose:          Print a sample story, using the fairytale of Jack and Jill as a framework
# Author:           Eric Pesch (ep3998)
# Date Modified:    2020-10-18

from typing import List
from dataclasses import dataclass
import random

@dataclass(frozen=True)
class Person:
    """Defines a character in our story
    """
    name: str

def fetch_water(person: Person) -> bool:
    """Randomly determine if water was fetched

    Args:
        person (Person): Character

    Returns:
        bool: If successful fetching water
    """
    return random.choice([True, False])


def story(characters: List[Person]) -> str:
    """Fairytale

    Args:
        characters (List[Person]): List of lead characters

    Returns:
        str: The result of the story
    """
    results = []

    # Choose a character to fetch water
    random.shuffle(characters)
    char_fetch_water = characters.pop()
    has_water = fetch_water(char_fetch_water)

    # Print story output, depending on water result
    if has_water:
        results.append(f"{char_fetch_water.name} fetched a pail of water.")
        results.extend([f"{char.name} felt overjoyed!" for char in characters])
    else:
        results.append(f"{char_fetch_water.name} fell down and broke their crown.")
        results.extend([f"{char.name} came tumbling after!" for char in characters])


    return "\n".join(results)


# List of characters in our story
story_chars = [
    Person("Jack"),
    Person("Jill"),
    Person("James")
]

# Print story output
print(story(story_chars))
